﻿local ResComm = LibStub:GetLibrary("LibResComm-1.0");
if not ResComm then return end

local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("Grid2")

local Grid2StatusRes = Grid2:NewModule("StatusRes")

local ResIncoming = Grid2.statusPrototype:new("resurrection-incoming")
local Resurrected = Grid2.statusPrototype:new("resurrected")
local Soulstone = Grid2.statusPrototype:new("soulstone")

Grid2 = Grid2
local DBL = LibStub:GetLibrary("LibDBLayers-1.0")

--{{{ AceBucket-3.0, embedded upon use

function Grid2StatusRes:RegisterBucketEvent(...)
	LibStub("AceBucket-3.0"):Embed(Grid2StatusRes)
	return self:RegisterBucketEvent(...)
end

function Grid2StatusRes:RegisterBucketMessage(...)
	LibStub("AceBucket-3.0"):Embed(Grid2StatusRes)
	return self:RegisterBucketMessage(...)
end

--}}}

do
	local count = 0
	function Grid2StatusRes:EnableResComm(enable)
		local prev = (count == 0)
		if enable then
			count = count + 1
		else
			count = count - 1
		end
		assert(count >= 0)
		local curr = (count == 0)
		if prev ~= curr then
			if curr then
				ResComm.UnregisterCallback(self, "ResComm_ResStart")
				ResComm.UnregisterCallback(self, "ResComm_ResEnd")
				ResComm.UnregisterCallback(self, "ResComm_Ressed")
				ResComm.UnRegisterCallback(self, "ResComm_CanRes")
				ResComm.UnRegisterCallback(self, "ResComm_ResExpired")
				self:UnregisterBucketEvent("UNIT_HEALTH")
			else
				ResComm.RegisterCallback(self, "ResComm_ResStart")
				ResComm.RegisterCallback(self, "ResComm_ResEnd")
				ResComm.RegisterCallback(self, "ResComm_Ressed")
				ResComm.RegisterCallback(self, "ResComm_CanRes")
				ResComm.RegisterCallback(self, "ResComm_ResExpired")
				self:RegisterBucketEvent("UNIT_HEALTH", 0.1, "HealthChanged")
			end
		end
	end
end



local unitResIncoming = {}

function ResIncoming:OnEnable()
	Grid2StatusRes:EnableResComm(true)
end

function ResIncoming:OnDisable()
	Grid2StatusRes:EnableResComm(false)
end

function ResIncoming:IsActive(unit)
	return unitResIncoming[unit]
end

function ResIncoming:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function ResIncoming:GetText(unit)
	return L["+RES"]
end

local function CreateResIncoming(baseKey, dbx)
	Grid2:RegisterStatus(ResIncoming, {"color", "text"}, baseKey, dbx)

	return ResIncoming
end

Grid2.setupFunc["resurrection-incoming"] = CreateResIncoming



local unitResurrected = {}

function Resurrected:OnEnable()
	Grid2StatusRes:EnableResComm(true)
end

function Resurrected:OnDisable()
	Grid2StatusRes:EnableResComm(false)
end

function Resurrected:IsActive(unit)
	return unitResurrected[unit]
end

function Resurrected:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function Resurrected:GetText(unit)
	return L["RES"]
end

local function CreateResurrected(baseKey, dbx)
	Grid2:RegisterStatus(Resurrected, {"color", "text"}, baseKey, dbx)

	return Resurrected
end

Grid2.setupFunc["resurrected"] = CreateResurrected



local unitSoulstone = {}

function Soulstone:OnEnable()
	Grid2StatusRes:EnableResComm(true)
end

function Soulstone:OnDisable()
	Grid2StatusRes:EnableResComm(false)
end

function Soulstone:IsActive(unit)
	return unitSoulstone[unit]
end

function Soulstone:GetColor(unit)
	local color = self.dbx.color1
	return color.r, color.g, color.b, color.a
end

function Soulstone:GetText(unit)
	return L["SS"]
end

local function CreateSoulstone(baseKey, dbx)
	Grid2:RegisterStatus(Soulstone, {"color", "text"}, baseKey, dbx)

	return Soulstone
end

Grid2.setupFunc["soulstone"] = CreateSoulstone



function Grid2StatusRes:ResComm_ResStart(event, _, _, fullName)
	local unit = Grid2:GetUnitByFullName(fullName)

	self:Debug("ResComm_ResStart fullName", fullName, "unit", unit)
	if (unit) then
		unitResIncoming[unit] = true
		if (ResIncoming.enabled) then ResIncoming:UpdateIndicators(unit) end
	else
		print("Grid2StatusRes:ResComm_ResStart nil unit. fullName:", fullName)
	end
end

function Grid2StatusRes:ResComm_ResEnd(event, _, fullName)
	if (fullName and not ResComm:IsUnitBeingRessed(fullName)) then
		local unit = Grid2:GetUnitByFullName(fullName)

		self:Debug("ResComm_ResEnd fullName", fullName, "unit", unit)
		if (unitResIncoming[unit]) then
			unitResIncoming[unit] = nil
			if (ResIncoming.enabled) then ResIncoming:UpdateIndicators(unit) end
		end
	end
end

function Grid2StatusRes:ResComm_ResExpired(event, fullName)
	local unit = Grid2:GetUnitByFullName(fullName)

	self:Debug("ResComm_ResExpired fullName", fullName, "unit", unit)
	if (unitResurrected[unit]) then
		unitResurrected[unit] = nil
		if (Resurrected.enabled) then Resurrected:UpdateIndicators(unit) end
	end
end

function Grid2StatusRes:ResComm_Ressed(event, fullName)
	local unit = Grid2:GetUnitByFullName(fullName)

	self:Debug("ResComm_Ressed fullName", fullName, "unit", unit)
	if (unit) then
		unitResurrected[unit] = true
		if (Resurrected.enabled) then Resurrected:UpdateIndicators(unit) end
	else
		print("Grid2StatusRes:ResComm_Ressed nil unit. fullName:", fullName)
	end
end

function Grid2StatusRes:ResComm_CanRes(event, fullName)
	local unit = Grid2:GetUnitByFullName(fullName)

	self:Debug("ResComm_ResStart fullName", fullName, "unit", unit)
	if (unit) then
		unitSoulstone[unit] = true
		if (Soulstone.enabled) then Soulstone:UpdateIndicators(unit) end
	else
		print("Grid2StatusRes:ResComm_CanRes nil unit. fullName:", fullName)
	end
end


function Grid2StatusRes:HealthChanged(units)
	for unit in pairs(units) do
		if (not UnitIsDead(unit)) then
			if (ResIncoming.enabled and unitResIncoming[unit]) then
				unitResIncoming[unit] = nil
				ResIncoming:UpdateIndicators(unit)
			end
			if (Resurrected.enabled and unitResurrected[unit]) then
				unitResurrected[unit] = nil
				Resurrected:UpdateIndicators(unit)
			end
			if (Soulstone.enabled and unitSoulstone[unit]) then
				unitSoulstone[unit] = nil
				Soulstone:UpdateIndicators(unit)
			end
		end
	end
end



-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(dblData, ...)
	local upgrade = prev_LoadOptions(self, dblData, ...)

	upgrade = DBL:LoadOptions("Grid2StatusResOptions", dblData, nil, "account", 1) or upgrade
	
	return upgrade
end

-- Hook UpgradeDefaults to blend in default options if current ones are old.
local prev_UpgradeDefaults = Grid2.UpgradeDefaults
function Grid2:UpgradeDefaults(dblData, ...)
	local flatten = prev_UpgradeDefaults(self, dblData, ...)
	
	local Grid2StatusResOptions = Grid2Options.plugins["Grid2StatusResOptions"]
	if (Grid2StatusResOptions) then
		flatten = DBL:UpgradeDefaults("Grid2StatusResOptions", dblData, Grid2StatusResOptions.UpgradeDefaults, "account", 1) or flatten
	end
	
	return flatten
end
